

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% vxinit.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initialization for VXMIN
%
% m-file called in vxmin.m
%

%%%%%%%%%%%%%
% set options
if nargin<3, options=[]; end;

n=size(x,1);

% convert to function handle if needed
fcn = fcnchk(fcn,0);

% start local time counter
tinit=tic;

% goal-oriented options:
%%%%%%%%%%%%%%%%%%%%%
%   .iscale         % initial scale: initially, the solution is sought 
%                   % in a box of radius iscale around xinit
%   .bounds         % bounds for random search
%     .lb              % lower bound
%     .ub              % upper bound
iscale=NaN;
if isfield(options,'iscale'), iscale=options.iscale; end;
if isnan(iscale), iscale=1; end;
iscale=max(realmin,iscale);
%%%%%%%%%%%%%%%%%%%%%
%   .stop           % stopping condition: stop if, for some k, 
%      .nftarget    % nf>=.nftarget(k) and
%      .ftarget     % fbest<=.ftarget(k),
%                   % but latest if nf>=max(nftarget)
%                   % to stop after nfmax function values, put
%                   % nftarget(1)=nfmax, ftarget(1)=inf;
%                   % to stop when fbest<=ftarget, put
%                   % nftarget(2)=0, ftarget(2)=ff;
%      .maxsec      % also stop after maxsec seconds
if isfield(options,'stop'), 
  nftarget=options.stop.nftarget;
  ftarget=options.stop.ftarget;
  [nfmax,imax]=max(nftarget);
  ftarget(imax)=inf;
  if isfield(options.stop,'maxsec'), maxsec=options.stop.maxsec;
  else                               maxsec=inf;
  end;
else,
  nfmax=4.8*n^2+200*n; 
  nftarget=nfmax;
  ftarget=inf;
  maxsec=inf;
end;
nfmax=max(2,nfmax);
%%%%%%%%%%%%%%%%%%%%%
%   .prt            % print level for progress info? 
%                   % (0=none, 1=little, 2=more)   
prt=NaN;
if isfield(options,'prt'), prt=options.prt; end;
if isnan(prt), prt=0; end;
%%%%%%%%%%%%%%%%%%%%%
%   .draw           % draw progress? (1=yes; assumes min f = 0)
draw=NaN;
if isfield(options,'draw'), draw=options.draw; end;
if isnan(draw), draw=0;end;
nmaxdraw=max(1,fix(nfmax/10));  % frequency of drawing
%%%%%%%%%%%%%%%%%%%%%
%   .newcounter     % new counter for progress drawing?
newcounter=NaN;
if isfield(options,'newcounter'), newcounter=options.newcounter; end;
if isnan(newcounter), newcounter=1; end;
%%%%%%%%%%%%%%%%%%%%%
%   .mark           % marking style (default: 'bo')   
mark=NaN;
if isfield(options,'mark'), mark=options.mark; end;
if isnan(mark), mark='bo';end;
%%%%%%%%%%%%%%%%%%%%%


% options for tuning performance:
%%%%%%%%%%%%%%%%%%%%%
%   .solver         % string defining the solver version
solver=NaN;
if isfield(options,'solver'), solver=options.solver; end;
if isnan(solver), solver='default'; end;
%%%%%%%%%%%%%%%%%%%%%
%   .gainfac        % factor for trial stepsize from expected gain 
gainfac=NaN;
if isfield(options,'gainfac'), gainfac=options.gainfac; end;
if isnan(gainfac), gainfac=2; end;
gainfac=max(eps,gainfac);
%%%%%%%%%%%%%%%%%%%%%
%   .discfac        % discount factor for expected gain
discfac=NaN;
if isfield(options,'discfac'), discfac=options.discfac; end;
if isnan(discfac), discfac=0.5; end;
discfac=max(eps,discfac);
%%%%%%%%%%%%%%%%%%%%%
%   .alptol         % tolerance in line search (defining first alp)
alptol=NaN;
if isfield(options,'alptol'), alptol=options.alptol; end;
if isnan(alptol), alptol=1e-2; end;
alptol=max(eps,alptol);
%%%%%%%%%%%%%%%%%%%%%
%   .alpeps         % tolerance in line search (defining lambdamin)
alpeps=NaN;
if isfield(options,'alpeps'), alpeps=options.alpeps; end;
if isnan(alpeps), alpeps=1e-12; end;
alpeps=max(eps,alpeps);
%%%%%%%%%%%%%%%%%%%%%
%   .regfac         % regularization factor for QR
regfac=NaN;
if isfield(options,'regfac'), regfac=options.regfac; end;
if isnan(regfac), regfac=1e-4; end;
regfac=max(n*eps,regfac);
%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%
% get bounds 
lb=options.bounds.lb;
ub=options.bounds.ub;


% now all options are set 
%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%
% initialization

if newcounter,
  % prepare display and function counter
  fobj=fcn;
  vxinitf(fobj,nfmax,prt,mark); % initialize function counter
  fcn=fcnchk('vxinitf',0);
  x=x(:);
end;

% initialize function value table
nflog=ceil(log10(nfmax));
flist=10.^[0:nflog];
flist=sort([flist,2*flist,5*flist]);
flist(flist>nfmax)=[];
flist(2,1:length(flist))=NaN;
dflist=inf(1,10); % list of previous gains in line searches


% initial best point information
nfused=0;
nfbest=nfused;xbest=x;fbest=inf;
done=~all(nfused<nftarget | fbest>ftarget);
ilist=1;
nfi=flist(1,ilist);
stat=[]; % no statistic yet
overcount=1.5; % overcounting factor
deltaf=1e6; % factor for weight in vxline.m
usestat=0; % create and use statistic only if explicitly activated
ns=n; % number of steps in statistical scout

%%%%%%%%%%%%%%%%%%%%%%%%%%
% try given starting point 
vxeval;  % function evaluation
fbest=f;

if draw, 
  semilogy(nfused,f,mark);hold on;drawnow
  xlabel(['nfmax=',num2str(nfmax)]);
  ndraw=nmaxdraw;
end;

% initial matrices
onn=ones(n,1);
X=xbest;          % affine basis
F=fbest;          % affine function value matrix

% initialization complete
%%%%%%%%%%%%%%%%%%%%%%%%%



