

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% vxqr_driver.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file illustrates the use of vxqr1.m
% by showing how to minimize the function f(x):=||Ax-b||_p^e
% for p=2,e=1 in dimension n=300 from a random starting point.
% The goal assumed in the example is the reduction of the initial 
% objective function value by a factor of 0.01 with at most nfmax=200*n 
% function evaluations, to return when one of the two limiting 
% conditions is first satisfied.
%
% A bounded initial search region must be specified but the actual 
% search is not confined to this region; thus the best point found 
% may be outside the search region.
%

clear;

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% create problem definition

% define problem parameters (to be adapted to your problem)
n=300;             % dimension
p=2;               % norm in objective function
e=1;               % exponent in objective function 

% create random matrix and right hand side for objective function
% (specific to the model problem; replace by whatever data you
% need to privide to your objectiv function)
A=rand(n)-0.5; 
x0=rand(n,1)-0.5;
x=A*x0;
b=-sum(A,2);

% create objective function f(x)=||Ax-b||_p^e
fcn=@(x) norm(A*x-b,p).^e; 
% To solve your own problem, simply replace in the previous line 
% the expression after @(x) by your expression or function call. 
% Parameters in this expression are passed by value, hence are 
% fixed during minimization.

x=2*rand(n,1)-1;      % starting point
ub=1e7*ones(n,1);     % upper bounds for search region
lb=-ub;               % lower bounds for search region
nfmax=200*n;          % stop after nfmax function evaluations
ftarget=0.01*fcn(x);  % quit when a point with f(x)<=ftarget is found
                      % (set ftarget=-inf if no target is known)
nsolve=5;             % number of solves with VXQR

% problem definition complete
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% translate into options
options.iscale=max(norm(lb,inf),norm(ub,inf));
options.stop.nftarget=[nfmax,0];
options.stop.ftarget=[inf,ftarget];
options.bounds.lb=lb;
options.bounds.ub=ub;


%%%%%%%%%%%%%%%%%%%%%%%%
% multiple calls to VXQR1
tic                    % set clock
nok=0;                 % number of problems solved so far
nfsum=0;               % number of function values used so far
for c=1:nsolve,

  % call solver
  disp(' ');disp('solving by VXQR1 ...');disp(' ');
  [xbest,fbest,nfused]=vxqr1(fcn,x,options);
  disp(' ');disp('... solved by VXQR1');

  % display output
  % xbest                   % best point found     
  fbest                     % function value at xbest    
  nfused                    % number of function evaluations used
  reduction=fbest/ftarget   % reduction factor 
                            % meaningful only if ftarget>0
  nok=nok+(reduction<=1);   % number of solved problems
  nfsum=nfsum+nfused;       % number of function values up to now
  seconds=toc               % time used up to now
end;
  
mean_seconds=seconds/nsolve   % average solution time
mean_funcost=nfsum/nsolve    % average number of function values used
success_rate=nok/nsolve       % succes rate


% solver calls complete
%%%%%%%%%%%%%%%%%%%%%%%